import pytest
import numpy as np
import discretisedfield as df
import micromagneticmodel as mm


class TestExchange:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.n = (10, 10, 6)
        self.subregions = {'r1': df.Region(p1=(-5e-9, -5e-9, -3e-9),
                                           p2=(5e-9, 0, 3e-9)),
                           'r2': df.Region(p1=(-5e-9, 0, -3e-9),
                                           p2=(5e-9, 5e-9, 3e-9))}

    def m_init(self, pos):
        x, y, z = pos
        if x <= 0:
            return (0, 0.5, 1)
        else:
            return (0, -0.5, 1)

    def test_scalar(self):
        name = 'exchange_scalar'

        A = 1e-12
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        mesh = df.Mesh(region=self.region, n=self.n)
        system.m = df.Field(mesh, dim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.average) - Ms) < 1

        self.calculator.delete(system)

    def test_dict(self):
        name = 'exchange_dict'

        A = {'r1': 0, 'r2': 1e-12}
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        mesh = df.Mesh(region=self.region, n=self.n,
                       subregions=self.subregions)
        system.m = df.Field(mesh, dim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # A=0 region
        assert abs(np.linalg.norm(system.m['r1'].average) - Ms) > 1e3

        # A!=0 region
        assert abs(np.linalg.norm(system.m['r2'].average) - Ms) < 1

        self.calculator.delete(system)

    def test_field(self):
        name = 'exchange_field'

        def A_fun(pos):
            x, y, z = pos
            if x <= 0:
                return 1e-10  # for 0, OOMMF gives nan
            else:
                return 1e-12

        mesh = df.Mesh(region=self.region, n=self.n)
        A = df.Field(mesh, dim=1, value=A_fun)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        system.m = df.Field(mesh, dim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.average) - Ms) < 1

        self.calculator.delete(system)



from .exchange import TestExchange
from .dmi import TestDMI
from .uniaxialanisotropy import TestUniaxialAnisotropy
from .cubicanisotropy import TestCubicAnisotropy
from .demag import TestDemag
from .zeeman import TestZeeman
from .energy import TestEnergy
from .precession import TestPrecession
from .damping import TestDamping
from .zhangli import TestZhangLi
from .slonczewski import TestSlonczewski
from .dynamics import TestDynamics
from .mindriver import TestMinDriver
from .timedriver import TestTimeDriver
from .mesh import TestMesh
from .compute import TestCompute
from .info_file import test_info_file
from .multiple_drives import test_multiple_drives
from .stdprob3 import test_stdprob3
from .stdprob4 import test_stdprob4
from .stdprob5 import test_stdprob5



import os
import glob
import pytest
import numpy as np
import discretisedfield as df
import micromagneticmodel as mm


class TestMinDriver:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup(self):
        p1 = (0, 0, 0)
        p2 = (5e-9, 5e-9, 5e-9)
        n = (5, 5, 5)
        self.Ms = 1e6
        A = 1e-12
        H = (0, 0, 1e6)
        region = df.Region(p1=p1, p2=p2)
        self.mesh = df.Mesh(region=region, n=n)
        self.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
        self.m = df.Field(self.mesh, dim=3, value=(0, 1, 0), norm=self.Ms)

    def test_noevolver_nodriver(self):
        name = 'mindriver_noevolver_nodriver'

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1e-3

        self.calculator.delete(system)

    def test_evolver_nodriver(self):
        name = 'mindriver_evolver_nodriver'

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        evolver = self.calculator.CGEvolver(method='Polak-Ribiere')
        md = self.calculator.MinDriver(evolver=evolver)
        md.drive(system)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1e-3

        self.calculator.delete(system)

    def test_noevolver_driver(self):
        name = 'mindriver_noevolver_driver'

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        md = self.calculator.MinDriver(stopping_mxHxm=0.1)
        md.drive(system)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1e-3

        self.calculator.delete(system)

    def test_evolver_driver(self):
        name = 'mindriver_evolver_driver'

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        evolver = self.calculator.CGEvolver(method='Polak-Ribiere')
        md = self.calculator.MinDriver(evolver=evolver, stopping_mxHxm=0.1)
        md.drive(system)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1e-3

        self.calculator.delete(system)

    def test_output_files(self):
        name = 'mindriver_output_files'

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        md = self.calculator.MinDriver()
        md.drive(system, save=True, overwrite=True)

        dirname = os.path.join(f'{name}', f'drive-{system.drive_number-1}')
        assert os.path.exists(dirname)
        miffilename = os.path.join(dirname, f'{name}.mif')
        assert os.path.isfile(miffilename)
        omf_files = list(glob.iglob(os.path.join(dirname, '*.omf')))
        assert len(omf_files) == 2
        odt_files = list(glob.iglob(os.path.join(dirname, '*.odt')))
        assert len(odt_files) == 1
        omffilename = os.path.join(dirname, 'm0.omf')
        assert omffilename in omf_files

        self.calculator.delete(system)

    def test_wrong_evolver(self):
        system = mm.examples.macrospin()
        evolver = self.calculator.RungeKuttaEvolver()
        md = self.calculator.MinDriver(evolver=evolver)

        with pytest.raises(TypeError):
            md.drive(system)

        self.calculator.delete(system)



import pytest
import pkg_resources
import micromagnetictests.calculatortests
from .get_tests import get_tests


def test():
    return pytest.main(['-v', '--pyargs',
                        'micromagnetictests'])  # pragma: no cover


__version__ = pkg_resources.get_distribution(__name__).version
__dependencies__ = pkg_resources.require(__name__)



import micromagnetictests as mt


def test_version():
    assert isinstance(mt.__version__, str)
    assert '.' in mt.__version__


def test_dependencies():
    assert isinstance(mt.__dependencies__, list)
    assert len(mt.__dependencies__) > 0



import setuptools

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='micromagnetictests',
    version='0.1.3',
    description=('Micromagnetic tests for Ubermag calculators'),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://ubermag.github.io',
    author='Marijan Beg and Hans Fangohr',
    packages=setuptools.find_packages(),
    include_package_data=True,
    python_requires='>=3.6',
    install_requires=['micromagneticmodel==0.8.19',
                      'scipy>=1.4'],
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Intended Audience :: Education',
                 'Intended Audience :: Developers',
                 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: BSD License',
                 'Natural Language :: English',
                 'Operating System :: MacOS',
                 'Operating System :: Microsoft :: Windows',
                 'Operating System :: Unix',
                 'Programming Language :: Python :: 3 :: Only',
                 'Topic :: Scientific/Engineering :: Physics',
                 'Topic :: Scientific/Engineering :: Mathematics',
                 'Topic :: Scientific/Engineering :: Visualization']
)
